<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO; 
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Request;
use Teknisa\MAN\Services; 
use Teknisa\MAN\Utilities;

use Teknisa\MAN\Services\DoacaoCartaoService;

class DoacaoCartao extends CrudWithParameterBag {
    
    protected $salesMessage, 
              $parameterBag,
              $doacaoCartaoService,
              $SMTPservice;
              
    protected $dataSourceNameArray      = array (
        'filterCampanha'         => 'doacaoCartao_Campanha',
        'filterCampanhaSelect'   => 'doacaoCartao_CampanhaSelect',
        'filterProduto'          => 'doacaoCartao_Produto',
        'filterCartao'           => 'doacaoCartao_Cartao',
        'filterCartaoDoado'      => 'doacaoCartao_CartaoDoado'
    );
    protected $parameterBagColumnsArray = array (
        'filterCampanha'         => array('NRORG', 'IDCAMPANHA'),
        'filterCampanhaSelect'   => array('NRORG', 'IDCAMPANHA', 'CDCAMPANHA', 'TCDCAMPANHA'),
        'filterProduto'          => array('NRORG', 'CDCAMPANHA'),
        'filterCartao'           => array('NRORG')
    );
    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, DoacaoCartaoService $doacaoCartaoService, Utilities\SalesMessage $salesMessage, Services\SMTPManagement $SMTPservice,  Utilities\Util $utilService) {
        parent::__construct($dataSourceManager, $parameterBag);
        $this->doacaoCartaoService = $doacaoCartaoService;
        $this->salesMessage = $salesMessage;
        $this->SMTPservice = $SMTPservice;
        $this->utilService = $utilService;
    }
    
    public function findCampanha(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterCampanha'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterCampanha'];
        parent::find($request, $response);
    }
    
    public function findCampanhaSelect(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterCampanhaSelect'];
        $conditions = $this->utilService->getParamsAsArray($request);
         $dataSourceFilter = array();
        if(array_key_exists('CDCAMPANHA',$conditions)){
            $conditions['TCDCAMPANHA'] = '';
        }else{
            $conditions['CDCAMPANHA'] = '';
            $conditions['TCDCAMPANHA'] = 'T';
        }
        
        foreach($conditions as $key => $value) {
            $dataSourceFilter[] = array(
                "columnName" => $key,
                "operator"=> "=",
                "value"=> $value
            );
        }
        
        $newRequest = new DTO\Request\Filter(
            new FilterCriteria(
                $request->getFilterCriteria()->getDataSourceName(),
                $dataSourceFilter,
                $request->getFilterCriteria()->getPage(),
                $request->getFilterCriteria()->getPageSize()
            ),
            $request->getMethod(),
            $request->getRoutePath(),
            $request->getUserId()
        );
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterCampanhaSelect'];
        parent::find($newRequest, $response);
    }
    
    public function findProduto(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterProduto'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterProduto'];
        parent::find($request, $response);
    }
    
    public function findCartao(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterCartao'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterCartao'];
        parent::find($request, $response);
    }
    
    public function findCartaoDoado(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceNameArray['filterCartaoDoado'];
        $this->parameterBagColumns = $this->parameterBagColumnsArray['filterCartao'];
        parent::find($request, $response);
    }
    
    public function criarCartao(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $NRORG             = $row['NRORG'];
        $CDPRODUTO         = $row['CDPRODUTO'];
        $CARTAOINI         = $row['CARTAOINI'];
        $CARTAOFIM         = $row['CARTAOFIM'];
        $CDCAMPANHA        = $row['CDCAMPANHA']; 
        $NRSEQPRODCAMP     = $row['NRSEQPRODCAMP'];
        $cont = $this->doacaoCartaoService->saveCartao($CARTAOINI, $CARTAOFIM, $CDCAMPANHA, $CDPRODUTO, $NRSEQPRODCAMP, $NRORG, $response);
        if ($cont > 0)
        $response->addNotification(new Notification($cont. " cartão(ões) adicionado(s) com sucesso!", Notification::TYPE_SUCCESS));
    } 
    
    public function criaCartoesFidelidade(DTO\Request $request, DTO\Response $response){
        try {
	        $row = $this->utilService->getParamsAsArray($request);
            
	        $CARTAOINI  = $row[0]['CARTAOINI'];
	        $CARTAOFIM  = $row[0]['CARTAOFIM'];
            $EMAIL      = $row[0]['EMAIL'];
            $EMAIL      = strtolower($EMAIL);
	        $cartoes ="<br/>";
	        $cartoesAux = array();
    		
    		for ( $cont = $CARTAOINI; $cont <= $CARTAOFIM ; $cont++) {
    		    $CDIDCONSUMID =	str_pad($cont, 9, "0", STR_PAD_LEFT);
	        	$CDIDCONSUMID =  $this->doacaoCartaoService->criaDigitoVerificador($CDIDCONSUMID);
            	$cartoes .= $CDIDCONSUMID."<br/>";
            	$cartoesAux[] = $CDIDCONSUMID;
	        }
	        $subject  = $this->salesMessage->getMessage('104');
			$sender = $this->salesMessage->getMessage('102');
			$content =
				'<p>Criação de Cartões Fidelidade Odhen Manage.</p>'.
	            '<p>Se não requisitou este teste, ignore essa mensagem.<b><br>'.
	            '<p>Segue a lista de cartões válidos<b><br>'.
                $cartoes
			;
            
            $arrAux = array();
            $arrAux[] = 'EMAIL ENVIADO COM SUCESSO';
            $arrAux[] = '';
            $arrAux[] = 'Segue a lista de Cartões Válidos';
            $arrAux[] = '';
            
            
			$this->SMTPservice->sendEmail($EMAIL, $EMAIL, $sender, $subject, $content);
            $arr = array_merge($arrAux, $cartoesAux);
            $response->addDataSet(new DataSet('Cartoes',$arr));
		} catch (\Exception $e) {
            /* Send failure message to the main function. */
            $arr[0] = 'Não foi possível enviar o email, confira a configuração do SMTP';
            $response->addDataSet(new DataSet('Cartoes',$arr));
        }
    }
    
    public function findFidelidade (){
        //Função vazia para utilizar o dataSource em vez de usar restEngine
    }
    
    public function saveCartao(DTO\Request $request, DTO\Response $response){
        $row = $this->utilService->getParamsAsArray($request);  
        $NRORG             = $row[0]['NRORG'];
        $CDPRODUTO         = $row[0]['CDPRODUTO'];
        $CARTAOINI         = $row[0]['CARTAOINI'];
        $CARTAOFIM         = $row[0]['CARTAOFIM'];
        $CDCAMPANHA        = $row[0]['CDCAMPANHA']; 
        $NRSEQPRODCAMP     = $row[0]['NRSEQPRODCAMP'];
        $params = array(
            'CARTAOINI' => $CARTAOINI,
            'CARTAOFIM' => $CARTAOFIM,
            'CDCAMPANHA' => $CDCAMPANHA,
            'CDPRODUTO' => $CDPRODUTO,
            'NRSEQPRODCAMP' => $NRSEQPRODCAMP,
            'NRORG' => $NRORG);
        $result = $this->doacaoCartaoService->saveCartao($params, $response);
        $cont = $result['Contador'];
        $cartoes = $result['Cartoes'];
        if(!$cartoes){
            $error = array('Para realizar o processo de Cadastro de Consumidores para Doação é necessário que os campos Cliente Fidelidade, Tipo de Consumidor Fidelidade e Centro de Custo Fidelidade estejam preenchidos na parametrização Fidelidade, na tela Gerais.');
            return $response->addDataSet(new DataSet('Erro', $error));
        }
        $response->addDataSet(new DataSet('ListaError',$cartoes));
        
    }
    
    public function findProdutoBeneficio(DTO\Request $request, DTO\Response $response){
        $conditions = $this->utilService->getParamsAsArray($request);
        $NRORG = $conditions['NRORG'];
        $CDCAMPANHA = $conditions['CDCAMPANHA'];
        $params = array(
            ':NRORG' => $NRORG,
            ':CDCAMPANHA' => $CDCAMPANHA);
        $result = $this->doacaoCartaoService->findProdutoBeneficio($params);
        $response->addDataSet(new DataSet('findProdutoBeneficio',$result));
    }
    
    public function findBeneficioDisponivel(DTO\Request $request, DTO\Response $response){
        $conditions = $this->utilService->getParamsAsArray($request);
        $NRORG = $conditions['NRORG'];
        $CDCONSUMIDOR = $conditions['CDCONSUMIDOR'];
        $CDCLIENTE = $conditions['CDCLIENTE'];
        $params = array(
            ':NRORG' => $NRORG,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR,
            ':CDCLIENTE' => $CDCLIENTE);
        $result = $this->doacaoCartaoService->findBeneficioDisponivel($params);
        $response->addDataSet(new DataSet('findBeneficioDisponivel',$result));;
    }
    
    public function findBeneficioUtilizados(DTO\Request $request, DTO\Response $response){
        $row = $this->utilService->getParamsAsArray($request);
        $NRORG = $row['NRORG'];
        $CDCONSUMIDOR = $row['CDCONSUMIDOR'];
        $CDCLIENTE = $row['CDCLIENTE'];
        $params = array(
            ':NRORG' => $NRORG,
            ':CDCONSUMIDOR' => $CDCONSUMIDOR,
            ':CDCLIENTE' => $CDCLIENTE);
        $result = $this->doacaoCartaoService->findBeneficioUtilizados($params);
        $response->addDataSet(new DataSet('findBeneficioUtilizados',$result));;
    }
    
    public function testeCartao(){
        //Função vazia para utilizar o dataSource em vez de usar restEngine
    }
    
}    