<?php

namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\Controller\CrudWithParameterBag;

class RelTempoPermanenciaMesa extends CrudWithParameterBag {
    const dataSourceConsumidor  = 'orm00025_relTempoPermanenciaMesa';
    
    protected $parameterBagColumnsConsumidor = array('CDFILIAL', 'TCDLOJA', 'CDLOJA', 'DTINI', 'DTFIM', 'NRMESA', 'TNRMESA');
    
    public function __construct ($parameterBag, Manager $dataSourceManager, $utilService) {
        $this->parameterBag             = $parameterBag;
        $this->dataSourceManager        = $dataSourceManager;
        $this->utilService              = $utilService;
    }
    
    public function find (DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName      = self::dataSourceConsumidor;
        $this->parameterBagColumns = $this->parameterBagColumnsConsumidor;
        $conditions                = $this->utilService->getParamsAsArray($request);
        $dataSourceFilter          = self::prepareParameters($conditions);
        $requestDataSourceFilter = $request->getFilterCriteria()->getConditions();
        
        foreach($requestDataSourceFilter as $filter) {
            if($filter['operator'] === 'MAPPED_LIKE_ALL') {
                array_push($dataSourceFilter, $filter);
            }
        }
        
        $newRequest = new DTO\Request\Filter(
            new FilterCriteria(
                $this->dataSourceName,
                $dataSourceFilter,
                $request->getFilterCriteria()->getPage()
            ),
            $request->getMethod(),
            $request->getRoutePath(),
            $request->getUserId()
        );
        
        parent::find($newRequest, $response);
    }
    
    private function prepareParameters($conditions) {
        if(isset($conditions['DTENTRVENDA'])) {
            $DTINI = $conditions['DTENTRVENDA'][0];
            $DTFIM = $conditions['DTENTRVENDA'][1];
        } else {
            $DTFIM = $DTINI = NULL;
        }
        if(isset($conditions['CDFILIAL'])){
            $CDFILIAL = $conditions['CDFILIAL'];
            $TCDFILIAL = NULL;
            
        } else {
            $CDFILIAL = NULL;
            $TCDFILIAL = 'T';
        }
        
        if(isset($conditions['CDLOJA']) && !!$conditions['CDLOJA']){
            $CDLOJA = implode('_', $conditions['CDLOJA']);
            $TCDLOJA = NULL;
            
        } else {
            $CDLOJA = NULL;
            $TCDLOJA = 'T';
        }
        
        if(isset($conditions['NRMESA']) && !!$conditions['NRMESA']){
            $NRMESA = implode('_', $conditions['NRMESA']);
            $TNRMESA = NULL;
            
        } else {
            $NRMESA = NULL;
            $TNRMESA = 'T';
        }
        
        $parameters = array(
            'CDFILIAL'  => $CDFILIAL,
            'TCDLOJA'   => $TCDLOJA,
            'CDLOJA'    => $CDLOJA,
            'NRMESA'    => $NRMESA,
            'TNRMESA'   => $TNRMESA,
            'DTINI'     => $DTINI,
            'DTFIM'     => $DTFIM
        );
        
        $dataSourceFilter = array();
        
        $dataSourceFilter[0] = array(
            "columnName" => 'CDFILIAL',
            "operator"   => "=",
            "value"      => $parameters['CDFILIAL']
        ); 
        $dataSourceFilter[1] = array(
            "columnName" => 'TCDLOJA',
            "operator"   => "=",
            "value"      => $parameters['TCDLOJA']
        );
        $dataSourceFilter[2] = array(
            "columnName" => 'CDLOJA',
            "operator"   => "=",
            "value"      => $parameters['CDLOJA']
        );
        $dataSourceFilter[3] = array(
            "columnName" => 'NRMESA',
            "operator"   => "=",
            "value"      => $parameters['NRMESA']
        ); 
        $dataSourceFilter[4] = array(
            "columnName" => 'TNRMESA',
            "operator"   => "=",
            "value"      => $parameters['TNRMESA']
        );
        $dataSourceFilter[5] = array(
            "columnName" => 'DTINI',
            "operator"   => "=",
            "value"      => $parameters['DTINI']
        );
        $dataSourceFilter[6] = array(
            "columnName" => 'DTFIM',
            "operator"   => "=",
            "value"      => $parameters['DTFIM']
        );
        
        return $dataSourceFilter;
    }
}